﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using System.Web.Http.Controllers;
using RESTWebAPI.Models;


namespace RESTWebAPI.Controllers
{
    public class ProductsController : ApiController
    {

        #region Members
        private readonly ProductRepository repository;
        #endregion Members

        #region Constructors
        public ProductsController()
            : this(new ProductRepository())
        {
        }

        public ProductsController(ProductRepository repository)
        {
            this.repository = repository;
        }
        #endregion

        #region GET
        // GET /api/products
        public IQueryable<Product> Get()
        {
            return this.repository.Products.AsQueryable<Product>();
        }

        // GET /api/products/id
        public Product Get(int id)
        {
            Product product = this.repository.Products.Find(p => p.ID == id);
            if (product == null)
                throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.NotFound));
            return product;
        }
        #endregion GET

        #region POST
        public HttpResponseMessage Post(Product product)
        {
            if (ModelState.IsValid)
            {
				var prevProduct = this.repository.Products.FindLast(p => p.ID == product.ID);
				if (prevProduct != null)
                    throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.Conflict));

                this.repository.Create(product);

                // create response
                HttpResponseMessage response = Request.CreateResponse(HttpStatusCode.Created, product);
                // 201 Created should also return the location of the new resource.
                // As we are creating more than one we'll only add the location of the first.
				string uri = Url.Route(null, new { id = product.ID });
                response.Headers.Location = new Uri(Request.RequestUri, uri);
                return response;
            }
            throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.BadRequest));
        }
        #endregion

        #region PUT
        // PUT /api/products/5
        public HttpResponseMessage Put(int id, Product pr)
        {
            if (ModelState.IsValid)
            {
                if (this.repository.Update(id, pr))
                    /* Alternatively the response can be
                     * 200 OK if update was enacted correctly (same as 204 No Content)
                     * The response does not have to include a representation of the entity updated
                    return new HttpResponseMessage(HttpStatusCode.OK);
                     */ 
                    return new HttpResponseMessage(HttpStatusCode.NoContent);
                
                // if resource is not found it should be created
                this.repository.Create(pr);

                // create response
                HttpResponseMessage response = Request.CreateResponse(HttpStatusCode.Created, pr);
                // 201 Created should also return the location of the new resource.
                string uri = Url.Route(null, new { id = pr.ID });
                response.Headers.Location = new Uri(Request.RequestUri, uri);
                return response;
            }
            throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.BadRequest));
        }
        #endregion

        #region DELETE
        // DELETE /api/products/5
        public HttpResponseMessage Delete(int id)
        {
            if (ModelState.IsValid)
            {
                return Request.CreateResponse(HttpStatusCode.OK, this.repository.Delete(id));
                /* Alternatively Delete can also return 
                 * 202 Accepted if the delete action is accepted but not enacted
                 * 204 No Content if the delete action has been enacted but the response does not include the deleted entity(s)
                return Request.CreateResponse(HttpStatusCode.NoContent);
                return Request.CreateResponse(HttpStatusCode.Accepted);
                */
            }
            throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.BadRequest));
        }
        #endregion
    }
}